use std::{fs, path::PathBuf};

use serde::{Deserialize, Serialize};
use shared::Shared;

use crate::virtual_network::Ip;

#[derive(Debug, PartialEq, Clone, Default, Serialize, Deserialize)]
pub struct User {
    pub name: String,
    pub password: String,
    pub ip: Ip,
    pub ports: Vec<u16>,
}

impl Into<Shared<User>> for User {
    fn into(self) -> Shared<User> {
        Shared::new(self)
    }
}

impl User {
    pub fn new(name: String, ip_block: u8) -> Self {
        Self {
            name,
            password: "".to_string(),
            ip: Ip(ip_block, 0, 0, 0),
            ports: Vec::new()
        }
    }
    pub fn set_password(&mut self, password: &str) -> bool {
        if let Ok(hash) = bcrypt::hash(password, bcrypt::DEFAULT_COST) {
            self.password = hash;
            true
        } else {
            false
        }
    }
    pub fn verify_password(&self, password: &str) -> bool {
        if let Ok(ok) = bcrypt::verify(password, &self.password) {
            ok
        } else {
            false
        }
    }
    pub fn password_cost(&self) -> u32 {
        let parts: Vec<&str> = self.password.split("$").collect();
        if parts.len() < 2 {
            return 0;
        }
        if let Ok(cost) = parts[2].parse::<u32>() {
            cost
        } else {
            0
        }
    }
    pub fn open_port(&mut self, port: u16) -> bool {
        if self.ports.contains(&port) {
            return false;
        }
        self.ports.push(port);
        true
    }
    pub fn close_port(&mut self, port: u16) -> bool {
        if !self.ports.contains(&port) {
            return false;
        }
        let mut idx: usize = 0;
        for p in self.ports.iter() {
            if p == &port {
                break;
            }
            idx += 1;
        }
        self.ports.remove(idx);
        true
    }
    pub fn save(&self) -> bool {
        let p = PathBuf::from("data/users".to_string());
        let fp = PathBuf::from(format!("data/users/{}.json", self.ip.0));
        _ = fs::create_dir_all(p);
        if let Ok(content) = serde_json::to_string(self) {
            let e = fs::write(fp, content);
            e.is_ok()
        } else {
            false
        }
    }
    pub fn load(ip_block: u8) -> Option<Self> {
        let p = PathBuf::from("data/users".to_string());
        let fp = PathBuf::from(format!("data/users/{}.json", ip_block));
        _ = fs::create_dir_all(p);
        if let Ok(content) = fs::read_to_string(fp) {
            if let Ok(u) = serde_json::from_str::<Self>(&content) {
                return Some(u);
            }
        }
        None
    }
}